@@ -74,7 +74,7 @@ our %var_def;
 our %var_use;
 
 require Exporter;
-our @ISA = qw(Exporter);
+our @ISA = 'Exporter';
 my @most = qw(redefined forget rebuild callers callees
 	      var_defs var_uses
 	      var_apropos);
@@ -115,6 +115,7 @@ my %code = (intro => "i", used => "",
 	    subdef => "s", subused => "&",
 	    formdef => "f", meth => "->");
 
+=over 4
 
 =head2 Functions
 
@@ -259,6 +260,14 @@ sub load_pad {
 	    next unless class($valsv) eq "GV";
 	    # these pad GVs don't have corresponding names, so same @pad
 	    # array can be used without collisions
+
+            # XXX: for some reason, on 5.10 $valsv->STASH can be a
+            # B::SPECIAL, which doesn't have a name.
+
+            # XXX: this segfaults on 5.10 for some reason while
+            # traversing File::Find::contract_name from main
+
+            next if ref $valsv->STASH eq 'B::SPECIAL';
 	    $pad[$ix] = [$valsv->STASH->NAME, "*", $valsv->NAME];
 	}
     }
@@ -287,7 +296,6 @@ sub xref {
 	    xref($op->pmreplstart);
 	} else {
 	    no strict 'refs';
-#             print STDERR $opname;
 	    my $ppname = "pp_$opname";
 	    &$ppname($op) if defined(&$ppname);
 	}
